from kolyaklimkLab3.MyParser.Constants import nonetype, moduletype, codetype, celltype, functype, bldinfunctype, \
    smethodtype, \
    cmethodtype, mapproxytype, wrapdesctype, metdesctype, getsetdesctype, CODE_PROPS, UNIQUE_TYPES, TYPE_KW, SOURCE_KW, \
    CODE_KW, GLOBALS_KW, NAME_KW, DEFAULTS_KW, CLOSURE_KW, BASES_KW, DICT_KW, CLASS_KW, OBJECT_KW

from kolyaklimkLab3.MyParser.Parser import Parser
